@@ -22,7 +22,6 @@
 #include <glib.h>
 
 #include <cstddef> // for std::size_t and optionally std::ptrdiff_t
-#include <utility> // For std::move()
 #include <initializer_list>
 #include <iosfwd>
 #include <iterator>
@@ -38,7 +37,7 @@ const std::basic_string<char>::size_type std::basic_string<char>::npos = (std::b
 namespace Glib
 {
 
-class ustring;
+class GLIBMM_API ustring;
 
 //********** Glib::StdStringView and Glib::UStringView *************
 
@@ -309,20 +308,6 @@ gunichar get_unichar_from_std_iterator(std::string::const_iterator pos) G_GNUC_P
  *                                    12, ustring::format(std::hex, 16));
  * @endcode
  *
- * @par %Glib::ustring as key in unordered associative containers
- * @par
- * To use e.g. std::unordered_map<Glib::ustring, int> there must be
- * a std::hash<Glib::ustring> specialization. Since glibmm 2.78 there is
- * such a specialization in glibmm, but it's available only if you include
- * the @ref ustringhash "ustring_hash.h" file.
- * @code
- * #include <glibmm/ustring_hash.h>
- * @endcode
- * This file is not included by #include <glibmm.h>, thereby saving some users
- * from an unpleasant surprise when they upgrade to glibmm 2.78 or later.
- * (If you have defined your own specialization of std::hash<Glib::ustring>,
- * the definition in glibmm/ustring_hash.h may clash with your definition.)
- *
  * @par Implementation notes
  * @par
  * %Glib::ustring does not inherit from std::string, because std::string was
@@ -332,7 +317,7 @@ gunichar get_unichar_from_std_iterator(std::string::const_iterator pos) G_GNUC_P
  * reimplement the interface so that all operations are based on characters
  * instead of bytes.
  */
-class ustring
+class GLIBMM_API ustring
 {
 public:
   using size_type = std::string::size_type;
@@ -363,83 +348,83 @@ public:
 #endif /* GLIBMM_HAVE_SUN_REVERSE_ITERATOR */
 
 #ifdef GLIBMM_HAVE_ALLOWS_STATIC_INLINE_NPOS
-  GLIBMM_API static const size_type npos = std::string::npos;
+  static const size_type npos = std::string::npos;
 #else
   // The IRIX MipsPro compiler says "The indicated constant value is not known",
   // so we need to initalize the static member data elsewhere.
-  GLIBMM_API static const size_type npos;
+  static const size_type npos;
 #endif
 
   /*! Default constructor, which creates an empty string.
    */
-  GLIBMM_API ustring();
+  ustring();
 
-  GLIBMM_API ~ustring() noexcept;
+  ~ustring() noexcept;
 
   /*! Construct a ustring as a copy of another ustring.
    * @param other A source string.
    */
-  GLIBMM_API ustring(const ustring& other);
+  ustring(const ustring& other);
 
   /*! Construct a ustring by moving from another ustring.
    * @param other A source string.
    */
-  GLIBMM_API ustring(ustring&& other);
+  ustring(ustring&& other);
 
   /*! Assign the value of another string by copying to this string.
    * @param other A source string.
    */
-  GLIBMM_API ustring& operator=(const ustring& other);
+  ustring& operator=(const ustring& other);
 
   /*! Assign the value of another string by moving to this string.
    * @param other A source string.
    */
-  GLIBMM_API ustring& operator=(ustring&& other);
+  ustring& operator=(ustring&& other);
 
   /*! Swap contents with another string.
    * @param other String to swap with.
    */
-  GLIBMM_API void swap(ustring& other);
+  void swap(ustring& other);
 
   /*! Construct a ustring as a copy of a std::string.
    * @param src A source <tt>std::string</tt> containing text encoded as UTF-8.
    */
-  GLIBMM_API ustring(const std::string& src);
+  ustring(const std::string& src);
 
   /*! Construct a ustring by moving from a std::string.
    * @param src A source <tt>std::string</tt> containing text encoded as UTF-8.
    */
-  GLIBMM_API ustring(std::string&& src);
+  ustring(std::string&& src);
 
   /*! Construct a ustring as a copy of a substring.
    * @param src %Source ustring.
    * @param i Index of first character to copy from.
    * @param n Number of UTF-8 characters to copy (defaults to copying the remainder).
    */
-  GLIBMM_API ustring(const ustring& src, size_type i, size_type n = npos);
+  ustring(const ustring& src, size_type i, size_type n = npos);
 
   /*! Construct a ustring as a partial copy of a C string.
    * @param src %Source C string encoded as UTF-8.
    * @param n Number of UTF-8 characters to copy.
    */
-  GLIBMM_API ustring(const char* src, size_type n);
+  ustring(const char* src, size_type n);
 
   /*! Construct a ustring as a copy of a C string.
    * @param src %Source C string encoded as UTF-8.
    */
-  GLIBMM_API ustring(const char* src);
+  ustring(const char* src);
 
   /*! Construct a ustring as multiple characters.
    * @param n Number of characters.
    * @param uc UCS-4 code point to use.
    */
-  GLIBMM_API ustring(size_type n, gunichar uc);
+  ustring(size_type n, gunichar uc);
 
   /*! Construct a ustring as multiple characters.
    * @param n Number of characters.
    * @param c ASCII character to use.
    */
-  GLIBMM_API ustring(size_type n, char c);
+  ustring(size_type n, char c);
 
   /*! Construct a ustring as a copy of a range.
    * @param pbegin Start of range.
@@ -451,19 +436,19 @@ public:
   //! @name Assign new contents.
   //! @{
 
-  GLIBMM_API ustring& operator=(const std::string& src);
-  GLIBMM_API ustring& operator=(std::string&& src);
-  GLIBMM_API ustring& operator=(const char* src);
-  GLIBMM_API ustring& operator=(gunichar uc);
-  GLIBMM_API ustring& operator=(char c);
+  ustring& operator=(const std::string& src);
+  ustring& operator=(std::string&& src);
+  ustring& operator=(const char* src);
+  ustring& operator=(gunichar uc);
+  ustring& operator=(char c);
 
-  GLIBMM_API ustring& assign(const ustring& src);
-  GLIBMM_API ustring& assign(ustring&& src);
-  GLIBMM_API ustring& assign(const ustring& src, size_type i, size_type n);
-  GLIBMM_API ustring& assign(const char* src, size_type n);
-  GLIBMM_API ustring& assign(const char* src);
-  GLIBMM_API ustring& assign(size_type n, gunichar uc);
-  GLIBMM_API ustring& assign(size_type n, char c);
+  ustring& assign(const ustring& src);
+  ustring& assign(ustring&& src);
+  ustring& assign(const ustring& src, size_type i, size_type n);
+  ustring& assign(const char* src, size_type n);
+  ustring& assign(const char* src);
+  ustring& assign(size_type n, gunichar uc);
+  ustring& assign(size_type n, char c);
   template <class In>
   ustring& assign(In pbegin, In pend);
 
@@ -471,19 +456,19 @@ public:
   //! @name Append to the string.
   //! @{
 
-  GLIBMM_API ustring& operator+=(const ustring& src);
-  GLIBMM_API ustring& operator+=(const char* src);
-  GLIBMM_API ustring& operator+=(gunichar uc);
-  GLIBMM_API ustring& operator+=(char c);
-  GLIBMM_API void push_back(gunichar uc);
-  GLIBMM_API void push_back(char c);
+  ustring& operator+=(const ustring& src);
+  ustring& operator+=(const char* src);
+  ustring& operator+=(gunichar uc);
+  ustring& operator+=(char c);
+  void push_back(gunichar uc);
+  void push_back(char c);
 
-  GLIBMM_API ustring& append(const ustring& src);
-  GLIBMM_API ustring& append(const ustring& src, size_type i, size_type n);
-  GLIBMM_API ustring& append(const char* src, size_type n);
-  GLIBMM_API ustring& append(const char* src);
-  GLIBMM_API ustring& append(size_type n, gunichar uc);
-  GLIBMM_API ustring& append(size_type n, char c);
+  ustring& append(const ustring& src);
+  ustring& append(const ustring& src, size_type i, size_type n);
+  ustring& append(const char* src, size_type n);
+  ustring& append(const char* src);
+  ustring& append(size_type n, gunichar uc);
+  ustring& append(size_type n, char c);
   template <class In>
   ustring& append(In pbegin, In pend);
 
@@ -491,17 +476,17 @@ public:
   //! @name Insert into the string.
   //! @{
 
-  GLIBMM_API ustring& insert(size_type i, const ustring& src);
-  GLIBMM_API ustring& insert(size_type i, const ustring& src, size_type i2, size_type n);
-  GLIBMM_API ustring& insert(size_type i, const char* src, size_type n);
-  GLIBMM_API ustring& insert(size_type i, const char* src);
-  GLIBMM_API ustring& insert(size_type i, size_type n, gunichar uc);
-  GLIBMM_API ustring& insert(size_type i, size_type n, char c);
+  ustring& insert(size_type i, const ustring& src);
+  ustring& insert(size_type i, const ustring& src, size_type i2, size_type n);
+  ustring& insert(size_type i, const char* src, size_type n);
+  ustring& insert(size_type i, const char* src);
+  ustring& insert(size_type i, size_type n, gunichar uc);
+  ustring& insert(size_type i, size_type n, char c);
 
-  GLIBMM_API iterator insert(iterator p, gunichar uc);
-  GLIBMM_API iterator insert(iterator p, char c);
-  GLIBMM_API void insert(iterator p, size_type n, gunichar uc);
-  GLIBMM_API void insert(iterator p, size_type n, char c);
+  iterator insert(iterator p, gunichar uc);
+  iterator insert(iterator p, char c);
+  void insert(iterator p, size_type n, gunichar uc);
+  void insert(iterator p, size_type n, char c);
   template <class In>
   void insert(iterator p, In pbegin, In pend);
 
@@ -509,18 +494,18 @@ public:
   //! @name Replace sub-strings.
   //! @{
 
-  GLIBMM_API ustring& replace(size_type i, size_type n, const ustring& src);
-  GLIBMM_API ustring& replace(size_type i, size_type n, const ustring& src, size_type i2, size_type n2);
-  GLIBMM_API ustring& replace(size_type i, size_type n, const char* src, size_type n2);
-  GLIBMM_API ustring& replace(size_type i, size_type n, const char* src);
-  GLIBMM_API ustring& replace(size_type i, size_type n, size_type n2, gunichar uc);
-  GLIBMM_API ustring& replace(size_type i, size_type n, size_type n2, char c);
+  ustring& replace(size_type i, size_type n, const ustring& src);
+  ustring& replace(size_type i, size_type n, const ustring& src, size_type i2, size_type n2);
+  ustring& replace(size_type i, size_type n, const char* src, size_type n2);
+  ustring& replace(size_type i, size_type n, const char* src);
+  ustring& replace(size_type i, size_type n, size_type n2, gunichar uc);
+  ustring& replace(size_type i, size_type n, size_type n2, char c);
 
-  GLIBMM_API ustring& replace(iterator pbegin, iterator pend, const ustring& src);
-  GLIBMM_API ustring& replace(iterator pbegin, iterator pend, const char* src, size_type n);
-  GLIBMM_API ustring& replace(iterator pbegin, iterator pend, const char* src);
-  GLIBMM_API ustring& replace(iterator pbegin, iterator pend, size_type n, gunichar uc);
-  GLIBMM_API ustring& replace(iterator pbegin, iterator pend, size_type n, char c);
+  ustring& replace(iterator pbegin, iterator pend, const ustring& src);
+  ustring& replace(iterator pbegin, iterator pend, const char* src, size_type n);
+  ustring& replace(iterator pbegin, iterator pend, const char* src);
+  ustring& replace(iterator pbegin, iterator pend, size_type n, gunichar uc);
+  ustring& replace(iterator pbegin, iterator pend, size_type n, char c);
   template <class In>
   ustring& replace(iterator pbegin, iterator pend, In pbegin2, In pend2);
 
@@ -528,20 +513,20 @@ public:
   //! @name Erase sub-strings.
   //! @{
 
-  GLIBMM_API void clear();
-  GLIBMM_API ustring& erase(size_type i, size_type n = npos);
-  GLIBMM_API ustring& erase();
-  GLIBMM_API iterator erase(iterator p);
-  GLIBMM_API iterator erase(iterator pbegin, iterator pend);
+  void clear();
+  ustring& erase(size_type i, size_type n = npos);
+  ustring& erase();
+  iterator erase(iterator p);
+  iterator erase(iterator pbegin, iterator pend);
 
   //! @}
   //! @name Compare and collate.
   //! @{
 
-  GLIBMM_API int compare(UStringView rhs) const;
-  GLIBMM_API int compare(size_type i, size_type n, UStringView rhs) const;
-  GLIBMM_API int compare(size_type i, size_type n, const ustring& rhs, size_type i2, size_type n2) const;
-  GLIBMM_API int compare(size_type i, size_type n, const char* rhs, size_type n2) const;
+  int compare(UStringView rhs) const;
+  int compare(size_type i, size_type n, UStringView rhs) const;
+  int compare(size_type i, size_type n, const ustring& rhs, size_type i2, size_type n2) const;
+  int compare(size_type i, size_type n, const char* rhs, size_type n2) const;
 
   /*! Create a unique sorting key for the UTF-8 string.  If you need to
    * compare UTF-8 strings regularly, e.g. for sorted containers such as
@@ -554,93 +539,93 @@ public:
    * the current locale.  Converting both operands to UCS-4 is just the first
    * of several costly steps involved when comparing ustrings.  So be careful.
    */
-  GLIBMM_API std::string collate_key() const;
+  std::string collate_key() const;
 
   /*! Create a unique key for the UTF-8 string that can be used for caseless
    * sorting.  <tt>ustr.casefold_collate_key()</tt> results in the same string
    * as <tt>ustr.casefold().collate_key()</tt>, but the former is likely more
    * efficient.
    */
-  GLIBMM_API std::string casefold_collate_key() const;
+  std::string casefold_collate_key() const;
 
   //! @}
   //! @name Extract characters and sub-strings.
   //! @{
 
   /*! No reference return; use replace() to write characters. */
-  GLIBMM_API value_type operator[](size_type i) const;
+  value_type operator[](size_type i) const;
 
   /*! No reference return; use replace() to write characters. @throw std::out_of_range */
-  GLIBMM_API value_type at(size_type i) const;
+  value_type at(size_type i) const;
 
-  GLIBMM_API inline ustring substr(size_type i = 0, size_type n = npos) const;
+  inline ustring substr(size_type i = 0, size_type n = npos) const;
 
   //! @}
   //! @name Access a sequence of characters.
   //! @{
 
-  GLIBMM_API iterator begin();
-  GLIBMM_API iterator end();
-  GLIBMM_API const_iterator begin() const;
-  GLIBMM_API const_iterator end() const;
-  GLIBMM_API reverse_iterator rbegin();
-  GLIBMM_API reverse_iterator rend();
-  GLIBMM_API const_reverse_iterator rbegin() const;
-  GLIBMM_API const_reverse_iterator rend() const;
+  iterator begin();
+  iterator end();
+  const_iterator begin() const;
+  const_iterator end() const;
+  reverse_iterator rbegin();
+  reverse_iterator rend();
+  const_reverse_iterator rbegin() const;
+  const_reverse_iterator rend() const;
 
   /**
    * @newin{2,52}
    */
-  GLIBMM_API const_iterator cbegin() const;
+  const_iterator cbegin() const;
 
   /**
    * @newin{2,52}
    */
-  GLIBMM_API const_iterator cend() const;
+  const_iterator cend() const;
 
   //! @}
   //! @name Find sub-strings.
   //! @{
 
-  GLIBMM_API size_type find(const ustring& str, size_type i = 0) const;
-  GLIBMM_API size_type find(const char* str, size_type i, size_type n) const;
-  GLIBMM_API size_type find(const char* str, size_type i = 0) const;
-  GLIBMM_API size_type find(gunichar uc, size_type i = 0) const;
-  GLIBMM_API size_type find(char c, size_type i = 0) const;
+  size_type find(const ustring& str, size_type i = 0) const;
+  size_type find(const char* str, size_type i, size_type n) const;
+  size_type find(const char* str, size_type i = 0) const;
+  size_type find(gunichar uc, size_type i = 0) const;
+  size_type find(char c, size_type i = 0) const;
 
-  GLIBMM_API size_type rfind(const ustring& str, size_type i = npos) const;
-  GLIBMM_API size_type rfind(const char* str, size_type i, size_type n) const;
-  GLIBMM_API size_type rfind(const char* str, size_type i = npos) const;
-  GLIBMM_API size_type rfind(gunichar uc, size_type i = npos) const;
-  GLIBMM_API size_type rfind(char c, size_type i = npos) const;
+  size_type rfind(const ustring& str, size_type i = npos) const;
+  size_type rfind(const char* str, size_type i, size_type n) const;
+  size_type rfind(const char* str, size_type i = npos) const;
+  size_type rfind(gunichar uc, size_type i = npos) const;
+  size_type rfind(char c, size_type i = npos) const;
 
   //! @}
   //! @name Match against a set of characters.
   //! @{
 
-  GLIBMM_API size_type find_first_of(const ustring& match, size_type i = 0) const;
-  GLIBMM_API size_type find_first_of(const char* match, size_type i, size_type n) const;
-  GLIBMM_API size_type find_first_of(const char* match, size_type i = 0) const;
-  GLIBMM_API size_type find_first_of(gunichar uc, size_type i = 0) const;
-  GLIBMM_API size_type find_first_of(char c, size_type i = 0) const;
+  size_type find_first_of(const ustring& match, size_type i = 0) const;
+  size_type find_first_of(const char* match, size_type i, size_type n) const;
+  size_type find_first_of(const char* match, size_type i = 0) const;
+  size_type find_first_of(gunichar uc, size_type i = 0) const;
+  size_type find_first_of(char c, size_type i = 0) const;
 
-  GLIBMM_API size_type find_last_of(const ustring& match, size_type i = npos) const;
-  GLIBMM_API size_type find_last_of(const char* match, size_type i, size_type n) const;
-  GLIBMM_API size_type find_last_of(const char* match, size_type i = npos) const;
-  GLIBMM_API size_type find_last_of(gunichar uc, size_type i = npos) const;
-  GLIBMM_API size_type find_last_of(char c, size_type i = npos) const;
+  size_type find_last_of(const ustring& match, size_type i = npos) const;
+  size_type find_last_of(const char* match, size_type i, size_type n) const;
+  size_type find_last_of(const char* match, size_type i = npos) const;
+  size_type find_last_of(gunichar uc, size_type i = npos) const;
+  size_type find_last_of(char c, size_type i = npos) const;
 
-  GLIBMM_API size_type find_first_not_of(const ustring& match, size_type i = 0) const;
-  GLIBMM_API size_type find_first_not_of(const char* match, size_type i, size_type n) const;
-  GLIBMM_API size_type find_first_not_of(const char* match, size_type i = 0) const;
-  GLIBMM_API size_type find_first_not_of(gunichar uc, size_type i = 0) const;
-  GLIBMM_API size_type find_first_not_of(char c, size_type i = 0) const;
+  size_type find_first_not_of(const ustring& match, size_type i = 0) const;
+  size_type find_first_not_of(const char* match, size_type i, size_type n) const;
+  size_type find_first_not_of(const char* match, size_type i = 0) const;
+  size_type find_first_not_of(gunichar uc, size_type i = 0) const;
+  size_type find_first_not_of(char c, size_type i = 0) const;
 
-  GLIBMM_API size_type find_last_not_of(const ustring& match, size_type i = npos) const;
-  GLIBMM_API size_type find_last_not_of(const char* match, size_type i, size_type n) const;
-  GLIBMM_API size_type find_last_not_of(const char* match, size_type i = npos) const;
-  GLIBMM_API size_type find_last_not_of(gunichar uc, size_type i = npos) const;
-  GLIBMM_API size_type find_last_not_of(char c, size_type i = npos) const;
+  size_type find_last_not_of(const ustring& match, size_type i = npos) const;
+  size_type find_last_not_of(const char* match, size_type i, size_type n) const;
+  size_type find_last_not_of(const char* match, size_type i = npos) const;
+  size_type find_last_not_of(gunichar uc, size_type i = npos) const;
+  size_type find_last_not_of(char c, size_type i = npos) const;
 
   //! @}
   //! @name Retrieve the string's size.
@@ -649,79 +634,75 @@ public:
   /** Returns true if the string is empty. Equivalent to *this == "".
    * @result Whether the string is empty.
    */
-  GLIBMM_API bool empty() const;
+  bool empty() const;
 
   /** Returns the number of characters in the string, not including any null-termination.
    * @result The number of UTF-8 characters.
    *
    * @see bytes(), empty()
    */
-  GLIBMM_API size_type size() const;
+  size_type size() const;
 
   // We have length() as well as size(), because std::string has both.
 
   /** This is the same as size().
    */
-  GLIBMM_API size_type length() const;
+  size_type length() const;
 
   /** Returns the number of bytes in the string, not including any null-termination.
    * @result The number of bytes.
    *
    * @see size(), empty()
    */
-  GLIBMM_API size_type bytes() const;
+  size_type bytes() const;
 
   //! @}
   //! @name Change the string's size.
   //! @{
 
-  GLIBMM_API void resize(size_type n, gunichar uc);
-  GLIBMM_API void resize(size_type n, char c = '\0');
+  void resize(size_type n, gunichar uc);
+  void resize(size_type n, char c = '\0');
 
   //! @}
   //! @name Control the allocated memory.
   //! @{
 
-  GLIBMM_API size_type capacity() const;
-  GLIBMM_API size_type max_size() const;
-  GLIBMM_API void reserve(size_type n = 0);
+  size_type capacity() const;
+  size_type max_size() const;
+  void reserve(size_type n = 0);
 
   //! @}
   //! @name Get a per-byte representation of the string.
   //! @{
 
-  GLIBMM_API inline operator std::string() const; // e.g. std::string str = ustring();
-  GLIBMM_API inline const std::string& raw() const;
-  /*! Return the stored string, moved from the %ustring.
-   * @newin{2,74}
-   */
-  GLIBMM_API inline std::string release();
+  inline operator std::string() const; // e.g. std::string str = ustring();
+  inline const std::string& raw() const;
 
   // Not necessarily an ASCII char*. Use g_utf8_*() where necessary.
-  GLIBMM_API const char* data() const;
-  GLIBMM_API const char* c_str() const;
+  const char* data() const;
+  const char* c_str() const;
 
   /*! @return Number of copied @em bytes, not characters. */
-  GLIBMM_API size_type copy(char* dest, size_type n, size_type i = 0) const;
+  size_type copy(char* dest, size_type n, size_type i = 0) const;
 
   //! @}
   //! @name UTF-8 utilities.
   //! @{
 
   /*! Check whether the string is valid UTF-8. */
-  GLIBMM_API bool validate() const;
+  bool validate() const;
 
   /*! Check whether the string is valid UTF-8. */
-  GLIBMM_API bool validate(iterator& first_invalid);
+  bool validate(iterator& first_invalid);
 
   /*! Check whether the string is valid UTF-8. */
-  GLIBMM_API bool validate(const_iterator& first_invalid) const;
+  bool validate(const_iterator& first_invalid) const;
 
   /*! Return a copy that is a valid UTF-8 string replacing invalid bytes in the
    *  original with %Unicode replacement character (U+FFFD).
    *  If the string is valid, return a copy of it.
    */
-  GLIBMM_API ustring make_valid() const;
+  ustring make_valid() const;
 
   /*! Check whether the string is plain 7-bit ASCII. @par
    * Unlike any other ustring method, is_ascii() is safe to use on invalid
@@ -729,22 +710,10 @@ public:
    * ASCII either, therefore is_ascii() will just return @c false then.
    * @return Whether the string contains only ASCII characters.
    */
-  GLIBMM_API bool is_ascii() const;
+  bool is_ascii() const;
 
   /*! "Normalize" the %Unicode character representation of the string. */
-  GLIBMM_API ustring normalize(NormalizeMode mode = NormalizeMode::DEFAULT_COMPOSE) const;
-
-  /*! Cuts off the middle of the string.
-   *
-   * Preserves half of @a truncate_length characters at the beginning
-   * and half at the end.
-   *
-   * If the string is already short enough, this returns a copy of the string.
-   * If @a truncate_length is 0, an empty string is returned.
-   *
-   * @newin{2,78}
-   */
-  GLIBMM_API ustring truncate_middle(gsize truncate_length) const;
+  ustring normalize(NormalizeMode mode = NormalizeMode::DEFAULT_COMPOSE) const;
 
   //! @}
   //! @name Character case conversion.
@@ -757,21 +726,21 @@ public:
    * <tt>&quot;&szlig;&quot;</tt> will be replaced by two characters
    * <tt>"SS"</tt> because there is no capital <tt>&quot;&szlig;&quot;</tt>.
    */
-  GLIBMM_API ustring uppercase() const;
+  ustring uppercase() const;
 
   /*! Returns a new UTF-8 string with all characters characters converted to
    * their lowercase equivalent, while honoring the current locale.  The
    * resulting string may change in the number of bytes as well as in the
    * number of characters.
    */
-  GLIBMM_API ustring lowercase() const;
+  ustring lowercase() const;
 
   /*! Returns a caseless representation of the UTF-8 string.  The resulting
    * string doesn't correspond to any particular case, therefore the result
    * is only useful to compare strings and should never be displayed to the
    * user.
    */
-  GLIBMM_API ustring casefold() const;
+  ustring casefold() const;
 
   //! @}
   //! @name Message formatting.
@@ -780,7 +749,7 @@ public:
   /* Returns fmt as is, but checks for invalid references in the format string.
    * @newin{2,18}
    */
-  GLIBMM_API static inline ustring compose(const ustring& fmt);
+  static inline ustring compose(const ustring& fmt);
 
   /*! Substitute placeholders in a format string with the referenced arguments.
    *
@@ -929,7 +898,7 @@ public:
    *
    * @newin{2,62}
    */
-  GLIBMM_API static inline ustring sprintf(const ustring& fmt);
+  static inline ustring sprintf(const ustring& fmt);
 
   /*! Overload of sprintf() for a format string only, which returns it unchanged
    * and avoids creating a temporary ustring as the argument.
@@ -939,7 +908,7 @@ public:
    *
    * @newin{2,62}
    */
-  GLIBMM_API static inline ustring sprintf(const char* fmt);
+  static inline ustring sprintf(const char* fmt);
 
   //! @}
 
@@ -963,13 +932,13 @@ private:
   template <class T>
   class Stringify;
 
-  GLIBMM_API static ustring compose_private(const ustring& fmt, std::initializer_list<const ustring*> ilist);
+  static ustring compose_private(const ustring& fmt, std::initializer_list<const ustring*> ilist);
 
   class FormatStream;
 
   template<class T> static inline const T& sprintify(const T& arg);
-  GLIBMM_API static inline const char* sprintify(const ustring& arg);
-  GLIBMM_API static inline const char* sprintify(const std::string& arg);
+  static inline const char* sprintify(const ustring& arg);
+  static inline const char* sprintify(const std::string& arg);
 
 #endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
@@ -996,19 +965,19 @@ struct ustring::SequenceToString<In, gunichar> : public std::string
 };
 
 template <>
-struct ustring::SequenceToString<Glib::ustring::iterator, gunichar> : public std::string
+struct GLIBMM_API ustring::SequenceToString<Glib::ustring::iterator, gunichar> : public std::string
 {
-  GLIBMM_API SequenceToString(Glib::ustring::iterator pbegin, Glib::ustring::iterator pend);
+  SequenceToString(Glib::ustring::iterator pbegin, Glib::ustring::iterator pend);
 };
 
 template <>
-struct ustring::SequenceToString<Glib::ustring::const_iterator, gunichar> : public std::string
+struct GLIBMM_API ustring::SequenceToString<Glib::ustring::const_iterator, gunichar> : public std::string
 {
-  GLIBMM_API SequenceToString(Glib::ustring::const_iterator pbegin, Glib::ustring::const_iterator pend);
+  SequenceToString(Glib::ustring::const_iterator pbegin, Glib::ustring::const_iterator pend);
 };
 
 
-class ustring::FormatStream
+class GLIBMM_API ustring::FormatStream
 {
 public:
   // noncopyable
@@ -1024,19 +993,18 @@ private:
   StreamType stream_;
 
 public:
-  GLIBMM_API FormatStream();
-  GLIBMM_API ~FormatStream() noexcept;
+  FormatStream();
+  ~FormatStream() noexcept;
 
   template <class T>
   inline void stream(const T& value);
 
-  GLIBMM_API inline void stream(const char* value);
+  inline void stream(const char* value);
 
   // This overload exists to avoid the templated stream() being called for non-const char*.
-  GLIBMM_API inline void stream(char* value);
+  inline void stream(char* value);
 
-  // TODO: C++20: Replace const with &&
-  GLIBMM_API ustring to_string() const;
+  ustring to_string() const;
 };
 
 #endif /* DOXYGEN_SHOULD_SKIP_THIS */
@@ -1280,8 +1248,6 @@ ustring::substr(ustring::size_type i, ustring::size_type n) const
   return ustring(*this, i, n);
 }
 
-// TODO: When we can break ABI – replace with a const& overload returning const std::string&
-// to avoid silent copies, and a corresponding && overload to move string out of rvalue this
 inline ustring::operator std::string() const
 {
   return string_;
@@ -1293,12 +1259,6 @@ ustring::raw() const
   return string_;
 }
 
-inline std::string
-ustring::release()
-{
-  return std::move(string_);
-}
-
 template <class... Ts>
 inline // static
   ustring
@@ -1306,7 +1266,6 @@ inline // static
 {
   ustring::FormatStream buf;
   (buf.stream(args), ...);
-  // TODO: C++20: std::move(buf).to_string()
   return buf.to_string();
 }
 
