@@ -10,7 +10,7 @@
 #define GIOMM_MAJOR_VERSION 2
 
 /* Minor version number of giomm. */
-#define GIOMM_MINOR_VERSION 80
+#define GIOMM_MINOR_VERSION 68
 
 /* Micro version number of giomm. */
 #define GIOMM_MICRO_VERSION 0
@@ -26,10 +26,8 @@
 #ifdef GIOMM_DLL
 # if defined(GIOMM_BUILD)
 #  define GIOMM_API __declspec(dllexport)
-# elif !defined (__GNUC__)
+# else
 #  define GIOMM_API __declspec(dllimport)
-# else /* don't dllimport classes/methods on GCC/MinGW */
-#  define GIOMM_API
 # endif
 /* Build a static library or non-native-Windows library */
 #else
